# Quark 1.0.452 run at 2016-11-28 14:25:58.554098
module Quark
require "quark"
def self.mdk_discovery; MdkDiscovery; end
module MdkDiscovery
def self.protocol; Protocol; end
module Protocol
require "datawire-quark-core"
require_relative '../quark/reflect' # 0 ('quark',) ('mdk_discovery',)
require_relative '../mdk_discovery' # 0 () ('mdk_discovery',)
require_relative '../mdk_protocol' # 0 () ('mdk_discovery',)
require_relative '../mdk_runtime/actors' # 0 ('mdk_runtime',) ('mdk_discovery',)
require_relative '../mdk_runtime' # 0 () ('mdk_discovery',)
require_relative '../quark' # 0 () ('mdk_discovery',)


def self.DiscoClientFactory; DiscoClientFactory; end
##
# Create a Discovery service client using standard MDK env variables and
# register it with the MDK.
#

class DiscoClientFactory < ::DatawireQuarkCore::QuarkObject
    attr_accessor :wsclient
    extend ::DatawireQuarkCore::Static

    static mdk_discovery_protocol_DiscoClientFactory_ref: -> { nil }



    def initialize(wsclient)
        
        self.__init_fields__
        (self).wsclient = wsclient

        nil
    end




    def create(subscriber, runtime)
        
        return ::Quark.mdk_discovery.protocol.DiscoClient.new(subscriber, @wsclient, runtime)

        nil
    end

    def isRegistrar()
        
        return true

        nil
    end

    def _getClass()
        
        return "mdk_discovery.protocol.DiscoClientFactory"

        nil
    end

    def _getField(name)
        
        if ((name) == ("wsclient"))
            return (self).wsclient
        end
        return nil

        nil
    end

    def _setField(name, value)
        
        if ((name) == ("wsclient"))
            (self).wsclient = ::DatawireQuarkCore.cast(value) { ::Quark.mdk_protocol.WSClient }
        end

        nil
    end

    def __init_fields__()
        
        self.wsclient = nil

        nil
    end


end
DiscoClientFactory.unlazy_statics

def self.DiscoClient; DiscoClient; end
##
# A source of discovery information that talks to Datawire Discovery server.
#
# Also supports registering discovery information with the server.
#

class DiscoClient < ::DatawireQuarkCore::QuarkObject
    attr_accessor :_failurePolicyFactory, :_dispatcher, :_timeService, :_subscriber, :_wsclient, :registered, :dlog, :lastHeartbeat, :sock
    extend ::DatawireQuarkCore::Static

    static mdk_discovery_protocol_DiscoClient_ref: -> { nil }



    def initialize(disco_subscriber, wsclient, runtime)
        
        self.__init_fields__
        (self)._subscriber = disco_subscriber
        (self)._wsclient = wsclient
        (self)._wsclient.subscribe(self)
        (self)._failurePolicyFactory = ::DatawireQuarkCore.cast((runtime).dependencies.getService("failurepolicy_factory")) { ::Quark.mdk_discovery.FailurePolicyFactory }
        (self)._timeService = runtime.getTimeService()

        nil
    end




    def onStart(dispatcher)
        
        (self)._dispatcher = dispatcher

        nil
    end

    def onStop()
        
        self.shutdown()

        nil
    end

    def onMessage(origin, message)
        
        klass = (::Quark.quark.reflect.QuarkClass.get(::DatawireQuarkCore._getClass(message))).id
        if ((klass) == ("mdk_discovery.RegisterNode"))
            register = ::DatawireQuarkCore.cast(message) { ::Quark.mdk_discovery.RegisterNode }
            self._register((register).node)
            return
        end
        ::Quark.mdk_protocol._subscriberDispatch(self, message)

        nil
    end

    def onMessageFromServer(message)
        
        type = (::Quark.quark.reflect.QuarkClass.get(::DatawireQuarkCore._getClass(message))).id
        if ((type) == ("mdk_discovery.protocol.Active"))
            active = ::DatawireQuarkCore.cast(message) { ::Quark.mdk_discovery.protocol.Active }
            self.onActive(active)
            return
        end
        if ((type) == ("mdk_discovery.protocol.Expire"))
            expire = ::DatawireQuarkCore.cast(message) { ::Quark.mdk_discovery.protocol.Expire }
            self.onExpire(expire)
            return
        end

        nil
    end

    def onWSConnected(websocket)
        
        (self).sock = websocket
        (self).lastHeartbeat = (((self)._timeService.time()) * (1000.0)).round()
        self.heartbeat()

        nil
    end

    def onPump()
        
        rightNow = (((self)._timeService.time()) * (1000.0)).round()
        heartbeatInterval = (((((self)._wsclient).ttl) / (2.0)) * (1000.0)).round()
        if (((rightNow) - ((self).lastHeartbeat)) >= (heartbeatInterval))
            (self).lastHeartbeat = rightNow
            self.heartbeat()
        end

        nil
    end

    ##
    # Register a node with the remote Discovery server.

    def _register(node)
        
        service = (node).service
        if (!((@registered).key?(service)))
            (@registered)[service] = (::Quark.mdk_discovery.Cluster.new((self)._failurePolicyFactory))
        end
        (@registered)[service].add(node)
        if ((self)._wsclient.isConnected())
            self.active(node)
        end

        nil
    end

    def active(node)
        
        active = ::Quark.mdk_discovery.protocol.Active.new()
        (active).node = node
        (active).ttl = ((self)._wsclient).ttl
        (self)._dispatcher.tell(self, active.encode(), (self).sock)
        @dlog.info(("active ") + (node.toString()))

        nil
    end

    def expire(node)
        
        expire = ::Quark.mdk_discovery.protocol.Expire.new()
        (expire).node = node
        (self)._dispatcher.tell(self, expire.encode(), (self).sock)
        @dlog.info(("expire ") + (node.toString()))

        nil
    end

    def resolve(node)
        
        nil

        nil
    end

    def onActive(active)
        
        (self)._dispatcher.tell(self, ::Quark.mdk_discovery.NodeActive.new((active).node), (self)._subscriber)

        nil
    end

    def onExpire(expire)
        
        (self)._dispatcher.tell(self, ::Quark.mdk_discovery.NodeExpired.new((expire).node), (self)._subscriber)

        nil
    end

    ##
    # Send all registered services.

    def heartbeat()
        
        services = ::DatawireQuarkCore::List.new(((self).registered).keys)
        idx = 0
        while ((idx) < ((services).size)) do
            jdx = 0
            nodes = (((self).registered)[(services)[idx]]).nodes
            while ((jdx) < ((nodes).size)) do
                self.active((nodes)[jdx])
                jdx = (jdx) + (1)
            end
            idx = (idx) + (1)
        end

        nil
    end

    def shutdown()
        
        services = ::DatawireQuarkCore::List.new(((self).registered).keys)
        idx = 0
        while ((idx) < ((services).size)) do
            jdx = 0
            nodes = (((self).registered)[(services)[idx]]).nodes
            while ((jdx) < ((nodes).size)) do
                self.expire((nodes)[jdx])
                jdx = (jdx) + (1)
            end
            idx = (idx) + (1)
        end

        nil
    end

    def _getClass()
        
        return "mdk_discovery.protocol.DiscoClient"

        nil
    end

    def _getField(name)
        
        if ((name) == ("_failurePolicyFactory"))
            return (self)._failurePolicyFactory
        end
        if ((name) == ("_dispatcher"))
            return (self)._dispatcher
        end
        if ((name) == ("_timeService"))
            return (self)._timeService
        end
        if ((name) == ("_subscriber"))
            return (self)._subscriber
        end
        if ((name) == ("_wsclient"))
            return (self)._wsclient
        end
        if ((name) == ("registered"))
            return (self).registered
        end
        if ((name) == ("dlog"))
            return (self).dlog
        end
        if ((name) == ("lastHeartbeat"))
            return (self).lastHeartbeat
        end
        if ((name) == ("sock"))
            return (self).sock
        end
        return nil

        nil
    end

    def _setField(name, value)
        
        if ((name) == ("_failurePolicyFactory"))
            (self)._failurePolicyFactory = ::DatawireQuarkCore.cast(value) { ::Quark.mdk_discovery.FailurePolicyFactory }
        end
        if ((name) == ("_dispatcher"))
            (self)._dispatcher = ::DatawireQuarkCore.cast(value) { ::Quark.mdk_runtime.actors.MessageDispatcher }
        end
        if ((name) == ("_timeService"))
            (self)._timeService = ::DatawireQuarkCore.cast(value) { ::Quark.mdk_runtime.Time }
        end
        if ((name) == ("_subscriber"))
            (self)._subscriber = ::DatawireQuarkCore.cast(value) { ::Quark.mdk_runtime.actors.Actor }
        end
        if ((name) == ("_wsclient"))
            (self)._wsclient = ::DatawireQuarkCore.cast(value) { ::Quark.mdk_protocol.WSClient }
        end
        if ((name) == ("registered"))
            (self).registered = ::DatawireQuarkCore.cast(value) { ::Hash }
        end
        if ((name) == ("dlog"))
            (self).dlog = value
        end
        if ((name) == ("lastHeartbeat"))
            (self).lastHeartbeat = ::DatawireQuarkCore.cast(value) { ::Integer }
        end
        if ((name) == ("sock"))
            (self).sock = ::DatawireQuarkCore.cast(value) { ::Quark.mdk_runtime.actors.Actor }
        end

        nil
    end

    def __init_fields__()
        
        self._failurePolicyFactory = nil
        self._dispatcher = nil
        self._timeService = nil
        self._subscriber = nil
        self._wsclient = nil
        self.registered = ::Hash.new()
        self.dlog = ::Quark.quark._getLogger("discovery")
        self.lastHeartbeat = 0
        self.sock = nil

        nil
    end


end
DiscoClient.unlazy_statics

def self.Active; Active; end
class Active < ::Quark.mdk_protocol.Serializable
    attr_accessor :node, :ttl
    extend ::DatawireQuarkCore::Static

    static _json_type: -> { "active" }
    static mdk_discovery_protocol_Active_ref: -> { nil }



    def initialize()
        
        super()

        nil
    end




    def _getClass()
        
        return "mdk_discovery.protocol.Active"

        nil
    end

    def _getField(name)
        
        if ((name) == ("_json_type"))
            return ::Quark.mdk_discovery.protocol.Active._json_type
        end
        if ((name) == ("node"))
            return (self).node
        end
        if ((name) == ("ttl"))
            return (self).ttl
        end
        return nil

        nil
    end

    def _setField(name, value)
        
        if ((name) == ("_json_type"))
            ::Quark.mdk_discovery.protocol.Active._json_type = ::DatawireQuarkCore.cast(value) { ::String }
        end
        if ((name) == ("node"))
            (self).node = ::DatawireQuarkCore.cast(value) { ::Quark.mdk_discovery.Node }
        end
        if ((name) == ("ttl"))
            (self).ttl = ::DatawireQuarkCore.cast(value) { ::Float }
        end

        nil
    end

    def __init_fields__()
        
        super
        self.node = nil
        self.ttl = nil

        nil
    end


end
Active.unlazy_statics

def self.Expire; Expire; end
##
# Expire a node.

class Expire < ::Quark.mdk_protocol.Serializable
    attr_accessor :node
    extend ::DatawireQuarkCore::Static

    static _json_type: -> { "expire" }
    static mdk_discovery_protocol_Expire_ref: -> { nil }



    def initialize()
        
        super()

        nil
    end




    def _getClass()
        
        return "mdk_discovery.protocol.Expire"

        nil
    end

    def _getField(name)
        
        if ((name) == ("_json_type"))
            return ::Quark.mdk_discovery.protocol.Expire._json_type
        end
        if ((name) == ("node"))
            return (self).node
        end
        return nil

        nil
    end

    def _setField(name, value)
        
        if ((name) == ("_json_type"))
            ::Quark.mdk_discovery.protocol.Expire._json_type = ::DatawireQuarkCore.cast(value) { ::String }
        end
        if ((name) == ("node"))
            (self).node = ::DatawireQuarkCore.cast(value) { ::Quark.mdk_discovery.Node }
        end

        nil
    end

    def __init_fields__()
        
        super
        self.node = nil

        nil
    end


end
Expire.unlazy_statics

def self.Clear; Clear; end
##
# Expire all nodes.

class Clear < ::Quark.mdk_protocol.Serializable
    extend ::DatawireQuarkCore::Static

    static _json_type: -> { "clear" }
    static mdk_discovery_protocol_Clear_ref: -> { nil }



    def initialize()
        
        super()

        nil
    end




    def _getClass()
        
        return "mdk_discovery.protocol.Clear"

        nil
    end

    def _getField(name)
        
        if ((name) == ("_json_type"))
            return ::Quark.mdk_discovery.protocol.Clear._json_type
        end
        return nil

        nil
    end

    def _setField(name, value)
        
        if ((name) == ("_json_type"))
            ::Quark.mdk_discovery.protocol.Clear._json_type = ::DatawireQuarkCore.cast(value) { ::String }
        end

        nil
    end

    def __init_fields__()
        
        super

        nil
    end


end
Clear.unlazy_statics


require_relative '../datawire_mdk_md' # 0 () ('mdk_discovery',)

end # module Protocol
end # module MdkDiscovery
end # module Quark
