# Quark 1.0.452 run at 2016-11-28 14:25:58.554098
module Quark
require "quark"
def self.mdk_discovery; MdkDiscovery; end
module MdkDiscovery
def self.synapse; Synapse; end
module Synapse
require "datawire-quark-core"
require_relative '../quark/reflect' # 0 ('quark',) ('mdk_discovery',)
require_relative '../mdk_discovery' # 0 () ('mdk_discovery',)
require_relative '../mdk_protocol' # 0 () ('mdk_discovery',)
require_relative '../mdk_runtime/actors' # 0 ('mdk_runtime',) ('mdk_discovery',)
require_relative '../mdk_runtime' # 0 () ('mdk_discovery',)
require_relative '../mdk_runtime/files' # 0 ('mdk_runtime',) ('mdk_discovery',)
require_relative '../quark/concurrent' # 0 ('quark',) ('mdk_discovery',)


def self.Synapse; Synapse; end
##
# AirBnB Synapse discovery source.
#
# Reads Synapse-generated files (https://github.com/airbnb/synapse#file) and
# generates discovery events.
#
# Usage:
#
# m = mdk.init()
# m.registerDiscoverySource(Synapse('/path/to/synapse/files'))
# m.start()
#
# All resulting Node instances will have version '1.0' hardcoded, and an
# address of the form 'host:port'.
#
# The original object from Synpase will be attached as the Node's properties.
#

class Synapse < ::DatawireQuarkCore::QuarkObject
    attr_accessor :_directory_path, :_environment
    extend ::DatawireQuarkCore::Static

    static mdk_discovery_synapse_Synapse_ref: -> { nil }



    def initialize(directory_path, environment)
        
        self.__init_fields__
        (self)._directory_path = directory_path
        (self)._environment = environment

        nil
    end




    def create(subscriber, runtime)
        
        return ::Quark.mdk_discovery.synapse._SynapseSource.new(subscriber, (self)._directory_path, runtime, (self)._environment)

        nil
    end

    def isRegistrar()
        
        return false

        nil
    end

    def _getClass()
        
        return "mdk_discovery.synapse.Synapse"

        nil
    end

    def _getField(name)
        
        if ((name) == ("_directory_path"))
            return (self)._directory_path
        end
        if ((name) == ("_environment"))
            return (self)._environment
        end
        return nil

        nil
    end

    def _setField(name, value)
        
        if ((name) == ("_directory_path"))
            (self)._directory_path = ::DatawireQuarkCore.cast(value) { ::String }
        end
        if ((name) == ("_environment"))
            (self)._environment = ::DatawireQuarkCore.cast(value) { ::Quark.mdk_protocol.OperationalEnvironment }
        end

        nil
    end

    def __init_fields__()
        
        self._directory_path = nil
        self._environment = nil

        nil
    end


end
Synapse.unlazy_statics

def self._SynapseSource; SynapseSource; end
##
# Implementation of the Synapse discovery source.

class SynapseSource < ::DatawireQuarkCore::QuarkObject
    attr_accessor :subscriber, :directory_path, :files, :dispatcher, :environment
    extend ::DatawireQuarkCore::Static

    static mdk_discovery_synapse__SynapseSource_ref: -> { nil }



    def initialize(subscriber, directory_path, runtime, environment)
        
        self.__init_fields__
        (self).subscriber = subscriber
        (self).directory_path = directory_path
        (self).files = runtime.getFileService()
        (self).environment = environment

        nil
    end




    def onStart(dispatcher)
        
        (self).dispatcher = dispatcher
        (self).dispatcher.tell(self, ::Quark.mdk_runtime.files.SubscribeChanges.new((self).directory_path), (self).files)

        nil
    end

    ##
    # Convert '/path/to/service_name.json' to 'service_name'.

    def _pathToServiceName(filename)
        
        parts = ::DatawireQuarkCore.split(filename, "/")
        service = (parts)[((parts).size) - (1)]
        return (service)[(0)...(((service).size) - (5))]

        nil
    end

    ##
    # Send an appropriate update to the subscriber for this DiscoverySource.

    def _update(service, nodes)
        
        (self).dispatcher.tell(self, ::Quark.mdk_discovery.ReplaceCluster.new(service, (self).environment, nodes), (self).subscriber)

        nil
    end

    def onMessage(origin, message)
        
        typeId = (::Quark.quark.reflect.QuarkClass.get(::DatawireQuarkCore._getClass(message))).id
        service = nil
        if ((typeId) == ("mdk_runtime.files.FileContents"))
            contents = ::DatawireQuarkCore.cast(message) { ::Quark.mdk_runtime.files.FileContents }
            if (!(((contents).path).end_with?(".json")))
                return
            end
            service = self._pathToServiceName((contents).path)
            json = ::DatawireQuarkCore::JSONObject.parse((contents).contents)
            nodes = ::DatawireQuarkCore::List.new([])
            idx = 0
            while ((idx) < (json.size())) do
                entry = json.getListItem(idx)
                node = ::Quark.mdk_discovery.Node.new()
                (node).id = ::Quark.quark.concurrent.Context.runtime().uuid()
                (node).service = service
                (node).version = "1.0"
                host = entry.getObjectItem("host").getString()
                port = ((entry.getObjectItem("port").getNumber()).round()).to_s
                (node).address = ((host) + (":")) + (port)
                (nodes) << (node)
                idx = (idx) + (1)
            end
            self._update(service, nodes)
            return
        end
        if ((typeId) == ("mdk_runtime.files.FileDeleted"))
            deleted = ::DatawireQuarkCore.cast(message) { ::Quark.mdk_runtime.files.FileDeleted }
            service = self._pathToServiceName((deleted).path)
            self._update(service, ::DatawireQuarkCore::List.new([]))
            return
        end

        nil
    end

    def _getClass()
        
        return "mdk_discovery.synapse._SynapseSource"

        nil
    end

    def _getField(name)
        
        if ((name) == ("subscriber"))
            return (self).subscriber
        end
        if ((name) == ("directory_path"))
            return (self).directory_path
        end
        if ((name) == ("files"))
            return (self).files
        end
        if ((name) == ("dispatcher"))
            return (self).dispatcher
        end
        if ((name) == ("environment"))
            return (self).environment
        end
        return nil

        nil
    end

    def _setField(name, value)
        
        if ((name) == ("subscriber"))
            (self).subscriber = ::DatawireQuarkCore.cast(value) { ::Quark.mdk_runtime.actors.Actor }
        end
        if ((name) == ("directory_path"))
            (self).directory_path = ::DatawireQuarkCore.cast(value) { ::String }
        end
        if ((name) == ("files"))
            (self).files = ::DatawireQuarkCore.cast(value) { ::Quark.mdk_runtime.files.FileActor }
        end
        if ((name) == ("dispatcher"))
            (self).dispatcher = ::DatawireQuarkCore.cast(value) { ::Quark.mdk_runtime.actors.MessageDispatcher }
        end
        if ((name) == ("environment"))
            (self).environment = ::DatawireQuarkCore.cast(value) { ::Quark.mdk_protocol.OperationalEnvironment }
        end

        nil
    end

    ##
    # The Actor should begin shutting down.

    def onStop()
        
        nil

        nil
    end

    def __init_fields__()
        
        self.subscriber = nil
        self.directory_path = nil
        self.files = nil
        self.dispatcher = nil
        self.environment = nil

        nil
    end


end
SynapseSource.unlazy_statics


require_relative '../datawire_mdk_md' # 0 () ('mdk_discovery',)

end # module Synapse
end # module MdkDiscovery
end # module Quark
